@@ -23,7 +23,6 @@ from typing import TYPE_CHECKING, Any
 from urllib.parse import quote, urlparse, urlunparse
 
 from airflow.providers.common.compat.openlineage.check import require_openlineage_version
-from airflow.providers.snowflake.version_compat import AIRFLOW_V_3_0_PLUS
 from airflow.utils import timezone
 
 if TYPE_CHECKING:
@@ -109,60 +108,6 @@ def fix_snowflake_sqlalchemy_uri(uri: str) -> str:
     return urlunparse((parts.scheme, hostname, parts.path, parts.params, parts.query, parts.fragment))
 
 
-def _get_logical_date(task_instance):
-    # todo: remove when min airflow version >= 3.0
-    if AIRFLOW_V_3_0_PLUS:
-        dagrun = task_instance.get_template_context()["dag_run"]
-        return dagrun.logical_date or dagrun.run_after
-
-    if hasattr(task_instance, "logical_date"):
-        date = task_instance.logical_date
-    else:
-        date = task_instance.execution_date
-
-    return date
-
-
-def _get_dag_run_clear_number(task_instance):
-    # todo: remove when min airflow version >= 3.0
-    if AIRFLOW_V_3_0_PLUS:
-        dagrun = task_instance.get_template_context()["dag_run"]
-        return dagrun.clear_number
-    return task_instance.dag_run.clear_number
-
-
-# todo: move this run_id logic into OpenLineage's listener to avoid differences
-def _get_ol_run_id(task_instance) -> str:
-    """
-    Get OpenLineage run_id from TaskInstance.
-
-    It's crucial that the task_instance's run_id creation logic matches OpenLineage's listener implementation.
-    Only then can we ensure that the generated run_id aligns with the Airflow task,
-    enabling a proper connection between events.
-    """
-    from airflow.providers.openlineage.plugins.adapter import OpenLineageAdapter
-
-    # Generate same OL run id as is generated for current task instance
-    return OpenLineageAdapter.build_task_instance_run_id(
-        dag_id=task_instance.dag_id,
-        task_id=task_instance.task_id,
-        logical_date=_get_logical_date(task_instance),
-        try_number=task_instance.try_number,
-        map_index=task_instance.map_index,
-    )
-
-
-# todo: move this run_id logic into OpenLineage's listener to avoid differences
-def _get_ol_dag_run_id(task_instance) -> str:
-    from airflow.providers.openlineage.plugins.adapter import OpenLineageAdapter
-
-    return OpenLineageAdapter.build_dag_run_id(
-        dag_id=task_instance.dag_id,
-        logical_date=_get_logical_date(task_instance),
-        clear_number=_get_dag_run_clear_number(task_instance),
-    )
-
-
 def _get_parent_run_facet(task_instance):
     """
     Retrieve the ParentRunFacet associated with a specific Airflow task instance.
@@ -173,22 +118,39 @@ def _get_parent_run_facet(task_instance):
     """
     from openlineage.client.facet_v2 import parent_run
 
-    from airflow.providers.openlineage.conf import namespace
+    from airflow.providers.openlineage.plugins.macros import (
+        lineage_job_name,
+        lineage_job_namespace,
+        lineage_root_job_name,
+        lineage_root_run_id,
+        lineage_run_id,
+    )
 
-    parent_run_id = _get_ol_run_id(task_instance)
-    root_parent_run_id = _get_ol_dag_run_id(task_instance)
+    parent_run_id = lineage_run_id(task_instance)
+    parent_job_name = lineage_job_name(task_instance)
+    parent_job_namespace = lineage_job_namespace()
+
+    root_parent_run_id = lineage_root_run_id(task_instance)
+    rot_parent_job_name = lineage_root_job_name(task_instance)
+
+    try:  # Added in OL provider 2.9.0, try to use it if possible
+        from airflow.providers.openlineage.plugins.macros import lineage_root_job_namespace
+
+        root_parent_job_namespace = lineage_root_job_namespace(task_instance)
+    except ImportError:
+        root_parent_job_namespace = lineage_job_namespace()
 
     return parent_run.ParentRunFacet(
         run=parent_run.Run(runId=parent_run_id),
         job=parent_run.Job(
-            namespace=namespace(),
-            name=f"{task_instance.dag_id}.{task_instance.task_id}",
+            namespace=parent_job_namespace,
+            name=parent_job_name,
         ),
         root=parent_run.Root(
             run=parent_run.RootRun(runId=root_parent_run_id),
             job=parent_run.RootJob(
-                name=task_instance.dag_id,
-                namespace=namespace(),
+                name=rot_parent_job_name,
+                namespace=root_parent_job_namespace,
             ),
         ),
     )
@@ -199,6 +161,7 @@ def _run_single_query_with_hook(hook: SnowflakeHook, sql: str) -> list[dict]:
     with closing(hook.get_conn()) as conn:
         hook.set_autocommit(conn, False)
         with hook._get_cursor(conn, return_dictionaries=True) as cur:
+            cur.execute("ALTER SESSION SET STATEMENT_TIMEOUT_IN_SECONDS = 3;")  # only for this session
             cur.execute(sql)
             result = cur.fetchall()
         conn.commit()
@@ -232,25 +195,36 @@ def _get_queries_details_from_snowflake(
     if not query_ids:
         return {}
     query_condition = f"IN {tuple(query_ids)}" if len(query_ids) > 1 else f"= '{query_ids[0]}'"
+    # https://docs.snowflake.com/en/sql-reference/account-usage#differences-between-account-usage-and-information-schema
+    # INFORMATION_SCHEMA.QUERY_HISTORY has no latency, so it's better than ACCOUNT_USAGE.QUERY_HISTORY
+    # https://docs.snowflake.com/en/sql-reference/functions/query_history
+    # SNOWFLAKE.INFORMATION_SCHEMA.QUERY_HISTORY() function seems the most suitable function for the job,
+    # we get history of queries executed by the user, and we're using the same credentials.
     query = (
         "SELECT "
         "QUERY_ID, EXECUTION_STATUS, START_TIME, END_TIME, QUERY_TEXT, ERROR_CODE, ERROR_MESSAGE "
         "FROM "
-        "table(information_schema.query_history()) "
+        "table(snowflake.information_schema.query_history()) "
         f"WHERE "
         f"QUERY_ID {query_condition}"
         f";"
     )
 
     try:
-        # Can't import the SnowflakeSqlApiHook class and do proper isinstance check - circular imports
-        if hook.__class__.__name__ == "SnowflakeSqlApiHook":
-            result = _run_single_query_with_api_hook(hook=hook, sql=query)  # type: ignore[arg-type]
+        # Note: need to lazy import here to avoid circular imports
+        from airflow.providers.snowflake.hooks.snowflake_sql_api import SnowflakeSqlApiHook
+
+        if isinstance(hook, SnowflakeSqlApiHook):
+            result = _run_single_query_with_api_hook(hook=hook, sql=query)
             result = _process_data_from_api(data=result)
         else:
             result = _run_single_query_with_hook(hook=hook, sql=query)
     except Exception as e:
-        log.warning("OpenLineage could not retrieve extra metadata from Snowflake. Error encountered: %s", e)
+        log.info(
+            "OpenLineage encountered an error while retrieving additional metadata about SQL queries"
+            " from Snowflake. The process will continue with default values. Error details: %s",
+            e,
+        )
         result = []
 
     return {row["QUERY_ID"]: row for row in result} if result else {}
@@ -287,7 +261,7 @@ def _create_snowflake_event_pair(
     return start, end
 
 
-@require_openlineage_version(provider_min_version="2.3.0")
+@require_openlineage_version(provider_min_version="2.5.0")
 def emit_openlineage_events_for_snowflake_queries(
     task_instance,
     hook: SnowflakeHook | SnowflakeSqlApiHook | None = None,
@@ -416,8 +390,8 @@ def emit_openlineage_events_for_snowflake_queries(
         event_batch = _create_snowflake_event_pair(
             job_namespace=namespace(),
             job_name=f"{task_instance.dag_id}.{task_instance.task_id}.query.{counter}",
-            start_time=query_metadata.get("START_TIME", default_event_time),  # type: ignore[arg-type]
-            end_time=query_metadata.get("END_TIME", default_event_time),  # type: ignore[arg-type]
+            start_time=query_metadata.get("START_TIME", default_event_time),
+            end_time=query_metadata.get("END_TIME", default_event_time),
             # `EXECUTION_STATUS` can be `success`, `fail` or `incident` (Snowflake outage, so still failure)
             is_successful=query_metadata.get("EXECUTION_STATUS", default_state).lower() == "success",
             run_facets={**query_specific_run_facets, **common_run_facets, **additional_run_facets},
